@@ -13,39 +13,11 @@
 #include <QGridLayout>
 #include <QHBoxLayout>
 #include <QLabel>
-#include <QPainter>
 
 // TODO: I think it would actually be appropriate to use a std::bitset<64>
 //       for the internal representation of this class perhaps
 //       the only real caveat is the conversion to/from quint64
 
-//------------------------------------------------------------------------------
-// Name: paintEvent
-// Desc: draws the button
-//------------------------------------------------------------------------------
-void BitButton::paintEvent(QPaintEvent *)
-{
-    uint8_t alpha = 0x60;
-    QPainter painter(this);
-    painter.setRenderHint(QPainter::Antialiasing, true);
-    QPen pen(palette().text(), 1);
-    pen.setJoinStyle(Qt::MiterJoin);
-    painter.setPen(pen);
-
-    if (on_) {
-        painter.setBrush(palette().text());
-        alpha = 0xB0;
-    } else {
-        painter.setBrush(palette().base());
-    }
-
-    if (over_) {
-        painter.setBrush(QColor(palette().text().color().red(), palette().text().color().green(), palette().text().color().blue(), alpha));
-    }
-
-    painter.drawRect(rect().adjusted(1, 1, -1, -1));
-}
-
 //------------------------------------------------------------------------------
 // Name: KCalcBitset
 // Desc: constructor
@@ -57,7 +29,7 @@ KCalcBitset::KCalcBitset(QWidget *parent)
 {
     setFrameStyle(QFrame::Panel | QFrame::Sunken);
 
-    connect(bit_button_group_, qOverload<QAbstractButton *>(&QButtonGroup::buttonClicked), this, &KCalcBitset::slotToggleBit);
+    connect(bit_button_group_, &QButtonGroup::buttonClicked, this, &KCalcBitset::slotToggleBit);
 
     // smaller label font
     QFont fnt = font();
@@ -84,6 +56,7 @@ KCalcBitset::KCalcBitset(QWidget *parent)
                 auto const tmpBitButton = new BitButton(this);
                 tmpBitButton->setToolTip(i18n("Bit %1 = %2", bitCounter, 1ULL << bitCounter));
                 wordlayout->addWidget(tmpBitButton);
+                wordlayout->setStretch(bit, 1);
                 bit_button_group_->addButton(tmpBitButton, bitCounter);
                 bitCounter--;
             }
@@ -92,8 +65,24 @@ KCalcBitset::KCalcBitset(QWidget *parent)
             auto label = new QLabel(this);
             label->setText(QString::number(bitCounter + 1));
             label->setFont(fnt);
+            label->setMinimumSize(label->fontMetrics().size(Qt::TextSingleLine, QStringLiteral("56"))); // Make all labels have same size
             wordlayout->addWidget(label);
+            wordlayout->setStretch(8, 1);
         }
+        layout->setRowStretch(rows, 1);
+    }
+    
+    // layout stretch for columns
+    for (int cols = 0; cols < 4; cols++) {
+        layout->setColumnStretch(cols, 1);
+    }
+
+    // store current aspect ratio (using width:height)
+    QSize initialSize(size());
+    if (initialSize.height() != 0.0 && float(initialSize.width()) / float(initialSize.height()) < 2.5) {
+        ratio_ = float(initialSize.width()) / float(initialSize.height());
+    } else {
+        ratio_ = 1.355163727959698; // 538/397
     }
 }
 
@@ -141,3 +130,63 @@ void KCalcBitset::slotToggleBit(QAbstractButton *button)
         Q_EMIT valueChanged(value_);
     }
 }
+
+//------------------------------------------------------------------------------
+// Name: resizeEvent
+// Desc: make sure all bitButtons have the same size
+//------------------------------------------------------------------------------
+void KCalcBitset::resizeEvent(QResizeEvent *event)
+{
+    // Call the overridden resize event
+    QFrame::resizeEvent(event);
+
+    // Set our maximum size based on the space available in the parent (to keep aspect ratio)
+    QWidget *parent = parentWidget();
+    if (parent) {
+        QSize maxSize(parent->contentsRect().width(), parent->contentsRect().height());
+        if (maxSize.width() != 0 && maxSize.height() != 0) {
+            float actualRatio = float(maxSize.width()) / float(maxSize.height());
+
+            if (actualRatio > ratio_) {
+                // available space is too wide, limit width
+                maxSize.setWidth(ratio_ * maxSize.height());
+            } else if (actualRatio < ratio_) {
+                // available space is too tall, limit height
+                maxSize.setHeight(maxSize.width() / ratio_);
+            }
+
+            setMaximumSize(maxSize.width(), maxSize.height());
+        }
+    }
+
+    // Get the minimum size of all buttons
+    int minWidth = INT_MAX;
+    int minHeight = INT_MAX;
+    for (QObject *obj : bit_button_group_->buttons()) {
+        if (auto const button = qobject_cast<BitButton *>(obj)) {
+            minWidth = qMin(minWidth, button->rect().width());
+            minHeight = qMin(minHeight, button->rect().height());
+        }
+    }
+    
+    // If this worked, set the renderSize for all BitButtons
+    if (minWidth != INT_MAX && minHeight != INT_MAX) {
+        // Make sure the size is square
+        if (minWidth > minHeight)
+            minWidth = minHeight;
+        else if (minHeight > minWidth)
+            minHeight = minWidth;
+        
+        // Set it for all buttons
+        for (QObject *obj : bit_button_group_->buttons()) {
+            if (auto const button = qobject_cast<BitButton *>(obj)) {
+                QSize size = QSize(button->renderSize());
+                size.setWidth(minWidth);
+                size.setHeight(minHeight);
+                button->setRenderSize(size);
+            }
+        }
+    }
+
+    updateGeometry();
+}
